import { useReducer } from 'react';

interface State<T> {
  queue: T[];
}
interface Action<T> {
  type: 'enqueue' | 'dequeue' | 'clear';
  values?: T[];
}

function reducer<T>(state: State<T>, action: Action<T>) {
  switch (action.type) {
    case 'enqueue': {
      state.queue.push(...action.values!);
      break;
    }
    case 'dequeue': {
      state.queue.shift();
      break;
    }
    case 'clear': {
      return { queue: [] };
    }
    default:
      break;
  }
  return {
    queue: state.queue,
  };
}

export function useQueue<T>(arr: T[] = []) {
  const [state, dispatch] = useReducer(reducer, arr, (a) => ({
    queue: [...(a || [])],
  }));

  return {
    enqueue: (...values: T[]) => {
      dispatch({ type: 'enqueue', values });
      return values;
    },
    dequeue: () => {
      const item = state.queue[0] || null;
      dispatch({ type: 'dequeue' });
      return item;
    },
    clear: () => dispatch({ type: 'clear' }),
    forEach: (callback: (value: T, key: T, set: Set<T>) => void) =>
      // @ts-expect-error ignore
      state.set.forEach(callback),
    includes: (value: T) => state.queue.includes(value),
    peek: () => state.queue[0] || null,
    entries: state.queue.entries,
    values: state.queue.values,
    keys: state.queue.keys,
    size: state.queue.length,
  };
}
