import { useReducer } from 'react';

interface State<T> {
  stack: T[];
}
interface Action<T> {
  type: 'push' | 'pop' | 'clear';
  values?: T[];
}

function reducer<T>(state: State<T>, action: Action<T>) {
  switch (action.type) {
    case 'push': {
      state.stack.push(...action.values!);
      break;
    }
    case 'pop': {
      state.stack.pop();
      break;
    }
    case 'clear': {
      return { stack: [] };
    }
    default:
      break;
  }
  return {
    stack: state.stack,
  };
}

export function useStack<T>(arr: T[] = []) {
  const [state, dispatch] = useReducer(reducer, arr, (a) => ({
    stack: [...(a || [])],
  }));

  return {
    push: (...values: T[]) => {
      dispatch({ type: 'push', values });
      return values;
    },
    pop: () => {
      const item = state.stack[state.stack.length - 1] || null;
      dispatch({ type: 'pop' });
      return item;
    },
    clear: () => dispatch({ type: 'clear' }),
    forEach: (callback: (value: T, key: T, set: Set<T>) => void) =>
      // @ts-expect-error ignore
      state.set.forEach(callback),
    includes: (value: T) => state.stack.includes(value),
    peek: () => state.stack[state.stack.length - 1] || null,
    entries: state.stack.entries,
    values: state.stack.values,
    keys: state.stack.keys,
    size: state.stack.length,
  };
}
